// Requality: CKEditor plugin
// Implements functions used by the UniEditor component of the Requality project
// Details:
//   1. New toolbar button RequalitySave that saves changes and closes the editor.
//   2. New toolbar button RequalityExit that closes the editor without saving
//      changes (confirmation message is shown if changes are present).

// TODO: Calling requalitySave/requalityExit causes JS errors. Probably, editor
// instance is destroyed too early. Find out how to work around the problem!

(function () {
	// Command implementing RequalitySave action
	var saveCmd = {
		exec: function (editor) {
			var elem = editor.element.$;
			if ($(elem).hasClass('editable'))
				CKEDITOR.toolbarHidden = $('.cke_toolbox').is(':hidden');
			var new_data = editor.getData();
			editor.destroy(false);
			if ($(elem).html() == '')
				$(elem).html('&nbsp;');
			var pos = elem.id.indexOf('-');
			var field_type = elem.id.substr(0, pos);
			var uuid = elem.id.substr(pos + 1);
			if (field_type == 'tcid')
				changeNodeId(uuid, new_data);
			else if (field_type == 'reqname') {
				changeNodeAttr(uuid, '_name', new_data);
			}
			else if (field_type == 'tcdesc') {
				changeNodeAttr(uuid, '_description', new_data);
				changeNodeAttr(uuid, 'expectedResults', '');
			}
		}
	};
	// Command implementing RequalityExit action
	var exitCmd = {
		editorFocus: false,
		exec: function (editor) {
			if (editor.checkDirty())
				if (!confirm('Имеются несохранённые изменения. Подтверждаете их отмену?')) {
					return;
				}
			if ($(editor.element.$).hasClass('editable'))
				CKEDITOR.toolbarHidden = $('.cke_toolbox').is(':hidden');
			editor.destroy(true);
		}
	};

	// Register the new plugin named "requality".
	CKEDITOR.plugins.add('requality', {
		init: function (editor) {
			editor.addCommand('requalitySave', saveCmd);
			editor.addCommand('requalityExit', exitCmd);
			editor.ui.addButton('RequalitySave', {
				label: 'Сохранить изменения и закрыть редактор',
				icon: this.path + 'requalitysave.png',
				command: 'requalitySave'
			});
			editor.ui.addButton('RequalityExit', {
				label: 'Закрыть редактор без сохранения изменений',
				icon: this.path + 'requalityexit.png',
				command: 'requalityExit'
			});
		}
	});
})();
