/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.core;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.UUID;

public class MemoryStorage extends TreeStorage {
    private UUID root = null;
    private Map<UUID, String> toId = new HashMap<UUID, String>();
    private Map<UUID, Collection<UUID>> toChildren = new LinkedHashMap<UUID, Collection<UUID>>();
    private Map<UUID, Collection<Attribute>> toAttributes = new LinkedHashMap<UUID, Collection<Attribute>>();

    public void emulateDeltaCreate(UUID parentUUId, UUID uuid, String type,
            String id, Collection<Attribute> attributes) {
        toId.put(uuid, id);
        toChildren.put(uuid, new HashSet<UUID>());
        toAttributes.put(uuid, attributes);
        if (parentUUId == null) {
            root = uuid;
        } else {
            toChildren.get(parentUUId).add(uuid);
        }
        deltaTarget.created(uuid);
    }

    @Override
    public UUID getRootNode() {
        return root;
    }

    @Override
    public String getNodeId(UUID uuid) {
        return toId.get(uuid);
    }

    @Override
    public void createNode(UUID parentUUId, UUID uuid, String id,
            Collection<Attribute> attrs) {
        toId.put(uuid, id);
        toChildren.put(uuid, new HashSet<UUID>());
        if (attrs == null) {
            attrs = new HashSet<Attribute>();
        }
        toAttributes.put(uuid, attrs);
        if (parentUUId == null) {
            root = uuid;
        } else {
            toChildren.get(parentUUId).add(uuid);
        }
        deltaTarget.created(uuid);
    }

    @Override
    public void remove(UUID uuid) {
        toId.remove(uuid);
        toAttributes.remove(uuid);
//        assertEquals(0, toChildren.get(uuid).size());
        for(UUID puid: toChildren.keySet()) {
            if(toChildren.get(puid).contains(uuid)) {
                toChildren.get(puid).remove(uuid);
            }
        }
//        assertEquals(0, toChildren.get(uuid).size());
        toChildren.remove(uuid);
        deltaTarget.deleted(uuid);
    }

    @Override
    public Collection<Attribute> getAttributes(UUID uuid) {
        return toAttributes.get(uuid);
    }

    @Override
    public Collection<UUID> getChildren(UUID uuid) {
        return toChildren.get(uuid);
    }

    @Override
    public UUID getParent(UUID uuid) {
        for (UUID i : toChildren.keySet()) {
            if (toChildren.get(i).contains(uuid)) {
                return i;
            }
        }
        return null;
    }

    @Override
    public boolean canLoad(UUID uuid) {
        return toId.containsKey(uuid);
    }

    @Override
    public void changeNodeId(UUID uuid, String id) {
        toId.put(uuid, id);
        deltaTarget.changed(uuid);
    }

    @Override
    public void moveNodeTo(UUID node, UUID parent) {
        UUID oldParent = getParent(node);
        toChildren.get(oldParent).remove(node);
        toChildren.get(parent).add(node);
        deltaTarget.moved(node, oldParent, parent);
    }

    @Override
    public void saveAttributes(UUID node, Collection<Attribute> attributes) {
        toAttributes.put(node, attributes);
        
    }

    @Override
    public String getNodeResourcePath(UUID uuId) {
        // TODO Auto-generated method stub
        return null;
    }
}
