/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.core;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

public abstract class TreeStorage {
    protected IStorageDeltaListener deltaTarget;

    public TreeStorage() {
    }
    public void setDeltaTarget(IStorageDeltaListener deltaProcessor) {
        this.deltaTarget = deltaProcessor;
    }

    protected IStorageDeltaListener getDeltaTarget() {
        return deltaTarget;
    }

    public void createNode(UUID parentNode, UUID node, String type, String id) { 
        if(parentNode==null) {
            type = "DBRoot";
        }
        Set<Attribute> attr = new HashSet<Attribute>();
        attr.add(new Attribute(AttributeType.STRING, "_type",type));
        createNode(parentNode, node, id, attr);
    }

    public String getNodeType(UUID uuid) {
        for(Attribute a : getAttributes(uuid)) {
            if(a.getKey().equals("_type")) {
                return (String) a.getValue();
            }
        }
        return null;
    }

    abstract public UUID getRootNode();
    abstract public UUID getParent(UUID uuid);
    abstract public Collection<UUID> getChildren(UUID uuid);
    abstract public void createNode(UUID parentUUId, UUID uuid, String id, Collection<Attribute> attrs);
    abstract public void remove(UUID node);
    abstract public String getNodeId(UUID uuid);
    abstract public void changeNodeId(UUID node, String id);
    abstract public Collection<Attribute> getAttributes(UUID uuid);
    abstract public void saveAttributes(UUID node, Collection<Attribute> attributes);
    abstract public void moveNodeTo(UUID node, UUID parent);
    abstract public boolean canLoad(UUID uuid);
    abstract public String getNodeResourcePath(UUID uuId);
}
