package com.unitesk.requality.eclipse.editors.browser;

import java.io.File;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.handlers.report.OpenReportHandler;
import com.unitesk.requality.eclipse.tools.ITreeStateListener;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.report.Template;

// TODO: refacatoring
public class ReportViewer extends EditorPart {
    public static String ID = "com.unitesk.requality.editors.browser.ReportViewer";
    private ReportData rdata;
    private Browser browser;
    private String href;

    ITreeStateListener treeStateListener = new ITreeStateListener() {
        @Override
        public void open(TreeDB db) {
        }

        @Override
        public void delete(IProject proj) {
            if (TreesTracker.getResourceByNode(rdata).getProject().equals(proj)) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
            }
        }

        @Override
        public void close(TreeDB db) {
            if (rdata.getTreeDB().equals(db)) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
            }
        }
    };

    ITreeChangeListener treeChangeListener = new ITreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
        }

        @Override
        public void deleted(TreeNode node) {
            if (node.equals(rdata)) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
            }
        }

        @Override
        public void changed(TreeNode node) {
            // if(rdata)
            Template tpl = Activator.getDefault().getReportTemplates()
                    .get(rdata.getTemplateId());
            if (tpl == null) {
                // get first
                tpl = Activator.getDefault().getReportTemplates().values()
                        .iterator().next();
            }
            IResource res = OpenReportHandler.getReportFolder().getFile(
                    Path.fromOSString(tpl.getTemplates().get(0)
                            .getOutputFileName()));
            if (res.exists()) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        browser.refresh();
                    }
                });
            } else {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
            }
        }

        @Override
        public void added(TreeNode node) {
        }
    };

    public ReportViewer() {
    }

    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    @Override
    public void doSaveAs() {
    }

    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        setSite(site);
        setInput(input);
        try {
            rdata = (ReportData) ((NodeEditorInput) input).getStorage()
                    .getNode();
            Template tpl = Activator.getDefault().getReportTemplates()
                    .get(rdata.getTemplateId());
            if (tpl == null) {
                // get first
                tpl = Activator.getDefault().getReportTemplates().values()
                        .iterator().next();
            }
            String savePath = rdata.getSaveDir();
            href = savePath + File.separator
                    + tpl.getTemplates().get(0).getOutputFileName();
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    @Override
    public void createPartControl(Composite parent) {
        browser = new Browser(parent, SWT.MOZILLA);
        browser.setUrl(href);
        rdata.getTreeDB().addListener(treeChangeListener);
        TreesTracker.getTracker().addListener(treeStateListener);
    }

    @Override
    public void dispose() {
        rdata.getTreeDB().removeListener(treeChangeListener);
        TreesTracker.getTracker().removeListener(treeStateListener);
        super.dispose();
    }

    @Override
    public void setFocus() {
        browser.setFocus();
    }

}
