/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.editors.panels;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.cnf.RequalityLabelProvider;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodeDialog;
import com.unitesk.requality.eclipse.ui.dialogs.SelectReqDialog;
import com.unitesk.requality.nodetypes.Requirement;

public class BaseRequirementPanel extends AbstractPanel {
    private static final String EDIT_BASES = "Base requirements:";
    private static final int HEIGHT_MULT = 100;
    private static final String BUT_ADD = "Add...";
    private static final String BUT_REMOVE = "Remove...";
//    private static final String BUT_EDIT = "Edit...";
    static private String TITLE_ADD_BASE = "Add base requirement";
//    static private String TITLE_OPEN_BASE = "Open base requirement";
    static private String TITLE_REMOVE_BASE = "Remove base requirement";

    private TreeViewer basesTree;
    private Button addBaseButton = null;
    private Button removeBaseButton = null;

    private Menu basesMenu = null;
    private MenuItem removeBaseItem = null;
//    private MenuItem openBaseItem = null;

    
    private Requirement requirement;
    private List<String> baseReqIds= new ArrayList<String>();

    public void setRequirement(Requirement req) {
        requirement= req;
        if(isDisposed()) {
            return;
        }
        baseReqIds= new ArrayList<String>(req.getBaseNodes());
        basesTree.setInput(baseReqIds);
    }

    public BaseRequirementPanel(Composite parent, int style) {
        super(parent, style);
        
        setLayout(new GridLayout());
        new Label(this, SWT.NONE).setText(EDIT_BASES);

        Composite baseCont= new Composite( this, SWT.NONE);
        baseCont.setLayout(new GridLayout(2,false));
        baseCont.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
        basesTree= new TreeViewer( baseCont );
        GridData layautData= new GridData( GridData.FILL_HORIZONTAL );
        layautData.heightHint= HEIGHT_MULT;
        basesTree.getControl().setLayoutData( layautData );
        basesTree.setContentProvider(baseReqContentProvider);
        basesTree.setLabelProvider(new RequalityLabelProvider());
//        basesTree.addListener( SWT.MouseDoubleClick, openBaseLink );
//        basesTree.addListener( SWT.KeyDown, addBaseLink );
//        basesTree.addListener( SWT.KeyDown, openBaseLink );
        basesTree.getControl().addListener( SWT.KeyDown, removeBaseLink );
        
        // Создаем контекстное меню
        
        basesMenu= new Menu( basesTree.getControl() );
        basesTree.getControl().setMenu( basesMenu );
//        basesMenu.addListener( SWT.Show, baseMenuLink );
              
//        openBaseItem= new MenuItem( basesMenu, SWT.NONE );
//        openBaseItem.setText( TITLE_OPEN_BASE );
//        openBaseItem.addListener( SWT.Selection, openBaseLink );

        removeBaseItem= new MenuItem( basesMenu, SWT.NONE );
        removeBaseItem.setText( TITLE_REMOVE_BASE );
        removeBaseItem.addListener( SWT.Selection, removeBaseLink );

        Composite parsButtons= new Composite(baseCont, SWT.NONE);
        parsButtons.setLayout(new GridLayout());
        parsButtons.setLayoutData( new GridData( GridData.FILL_VERTICAL ) );

        // Создаем кнопку "Добавление"
        addBaseButton= new Button(parsButtons, SWT.PUSH);
        addBaseButton.setText(BUT_ADD);
        addBaseButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        addBaseButton.setToolTipText( TITLE_ADD_BASE );
        
        // Создаем кнопку "Удаление"
        removeBaseButton= new Button(parsButtons, SWT.PUSH); 
        removeBaseButton.setText(BUT_REMOVE);
        removeBaseButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        removeBaseButton.setToolTipText( TITLE_REMOVE_BASE );

        // Реакция на нажатие соответствующей кнопки
        addBaseButton.addListener( SWT.Selection, addBaseLink );
        removeBaseButton.addListener( SWT.Selection, removeBaseLink );

        // Реакция на нажатие клавиши "вниз"
//        addBaseButton.addListener( SWT.KeyDown, buttonLink );
//        removeBaseButton.addListener( SWT.KeyDown, buttonLink );
        initModifyListener();
    }

    protected void updateUI() {
        removeBaseButton.setEnabled(!basesTree.getSelection().isEmpty());
    }

    @Override
    protected void initModifyListener() {
        updateUI();
        basesTree.addSelectionChangedListener(new ISelectionChangedListener() {
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                updateUI();
            }
        });
    }

    @Override
    public boolean isChanged() {
        return ! requirement.getBaseNodes().equals(baseReqIds);
//        return false;
    }
    
    public List<String> getBaseReqIds() {
        return baseReqIds;
    }

    private Listener addBaseLink= new Listener() {
       public void handleEvent( Event event ) {
          if ( ( event.type == SWT.KeyDown ) && ( event.keyCode == SWT.INSERT )
                || ( event.type == SWT.Selection ) ) {
             SelectNodeDialog selDialog= new SelectReqDialog( getShell(), requirement.getTreeDB());
             if ( selDialog.open() == SelectNodeDialog.OK ) {
                Requirement selected= (Requirement) selDialog.getResult();
                if ( selected != null ) {
                    if(baseReqIds.contains(selected.getUUId())) {
                        MessageDialog.openInformation(getShell(), 
                                "Duplicate base requirement", 
                                "Base requirement is already in list");
                        return;
                    }

                    baseReqIds.add(selected.getQualifiedId());
                    basesTree.refresh();
                    BaseRequirementPanel.this.notifyListeners(SWT.Modify, new Event());
                }
             }
          }
       }
    };

    Listener removeBaseLink = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown) && (event.keyCode == SWT.DEL)
                    || (event.type == SWT.Selection)) {
                if (basesTree.getSelection() instanceof ITreeSelection) {
                    ITreeSelection treeSel = (ITreeSelection) basesTree.getSelection();
                    if (treeSel.getFirstElement() instanceof Requirement) {
                        Requirement selReq = (Requirement) treeSel.getFirstElement();
                        baseReqIds.remove(selReq.getQualifiedId());
                        BaseRequirementPanel.this.notifyListeners(SWT.Modify, new Event());
                        basesTree.refresh();
                        basesTree.getControl().setFocus();
                    }
                }
            }
        }
    };
    
    private ITreeContentProvider baseReqContentProvider= new ITreeContentProvider() {
        
        @Override
        public void inputChanged(Viewer arg0, Object arg1, Object arg2) {
            
        }
        
        @Override
        public void dispose() {
            
        }
        
        @Override
        public Object[] getElements(Object element) {
            return getChildren(element);
        }
        
        @Override
        public boolean hasChildren(Object element) {
            return false;
        }
        
        @Override
        public Object getParent(Object element) {
            return null;
        }
        
        @SuppressWarnings("unchecked")
        @Override
        public Object[] getChildren(Object parentElement) {
            if (parentElement instanceof List<?>) {
                List<String> listIds= (List<String>) parentElement;
                List<TreeNode> result= new ArrayList<TreeNode>();
                for(String baseId: listIds) { 
                    result.add(requirement.getTreeDB().getNode(baseId));
                }
                
                return result.toArray(new TreeNode[0]);
            }
            return null;
        }
    };
}
