/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.cnf;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;

public abstract class CNFAction extends Action implements IActionDelegate {

    protected ISelection selection;

    public CNFAction() {
    }

    @Override
    public void run(IAction action) {
        if (action instanceof CNFAction) {
            CNFAction a = (CNFAction) action;
            a.run();
        }
    }

    @Override
    public void selectionChanged(IAction action, ISelection selection) {
        if (action instanceof CNFAction) {
            CNFAction cnfa = (CNFAction) action;
            cnfa.setSelection(selection);
        }
    }

    private void setSelection(ISelection selection) {
        this.selection = selection;
    }

    public synchronized TreeNode getSelectedNode() {
        ISelection selection = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getSelectionService()
                .getSelection();
        if (!(selection instanceof ITreeSelection)) {
            return null;
        }
        ITreeSelection selected = (ITreeSelection) selection;

        if (selected.isEmpty()) {
            return null;
        }

        Object obj = selected.getFirstElement();
        if (obj instanceof TreeNode) {
            TreeNode node = (TreeNode) obj;
            return node;
        } else if (obj instanceof IFile) {
            IFile f = (IFile) obj;
            TreeDB db = TreesTracker.getTracker().getTree(f.getProject());
            ResourceStorage rs = (ResourceStorage) db.getStorage();
            if (rs.getUUId(f) == null) {
                return null;
            }
            return db.getNode(rs.getUUId(f));
        }
        return null;
    }

}