/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.cnf;

import java.util.UUID;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.Requirement;

public class RequalityLabelProvider extends LabelProvider implements ILabelProvider {

    @Override
    public Image getImage(Object element) {
        if (element instanceof IFile) {
            TreeDB db = TreesTracker.getTracker().getTree(
                    ((IFile) element).getProject());
            // TODO: more errors
            ResourceStorage rs = (ResourceStorage) db.getStorage();
            IFile f = (IFile) element;
            if (!f.exists()) {
                return PlatformUI.getWorkbench().getSharedImages()
                        .getImage(ISharedImages.IMG_DEC_FIELD_ERROR);
            }
            UUID uuid = rs.getUUId((IFile) element);
            if (uuid != null) {
                return getImage(db.getNode(uuid));
            }
        }

        if (element instanceof TreeDB) {
            return PlatformUI.getWorkbench().getSharedImages()
                    .getImage(IDE.SharedImages.IMG_OBJ_PROJECT);
        }

        if (element instanceof Location) {
            Location loc = (Location) element;
            if(loc.getRequirements().length==0) {
                return PlatformUI.getWorkbench().getSharedImages()
                        .getImage(ISharedImages.IMG_OBJS_ERROR_TSK);
            }
        }
        if (element instanceof TreeNode) {
            TreeNode node = (TreeNode) element;
            if (node.getQualifiedId().equals("/")) {
                return PlatformUI.getWorkbench().getSharedImages()
                        .getImage(IDE.SharedImages.IMG_OBJ_PROJECT);
            }
            Image result = Activator.getDefault().getImageRegistry()
                    .get("nodetype." + node.getType());
            if(result == null) {
                 return Activator.getDefault().getImageRegistry()
                        .get("nodetype.default");
            }
            return result;
        }
        return WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider().getImage(element);
    }

    @Override
    public String getText(Object element) {
        if (element instanceof IFile) {
            TreeDB db = TreesTracker.getTracker().getTree(
                    ((IFile) element).getProject());
            ResourceStorage rs = (ResourceStorage) db.getStorage();
            IFile f = (IFile) element;
            if (!f.exists()) {
                return "n/a";
            }
            UUID uuid = rs.getUUId(f);
            if (uuid != null) {
                return getText(db.getNode(uuid));
            }
        }

        if (element instanceof IProject) {
            TreeDB db = TreesTracker.getTracker().getTree(((IProject) element));
            if (db != null) {
                return ((IProject) element).getName();
            }
        }

        if (element instanceof TreeDB) {
            return TreesTracker.getTracker().getProject((TreeDB) element).getName();
        }
        if (element instanceof Requirement) {
            Requirement node = (Requirement) element;
            return node.getName();
        }
        if (element instanceof ReportData) {
            ReportData node = (ReportData) element;
            return node.getId();
        }
        if (element instanceof TreeNode) {
            TreeNode node = (TreeNode) element;
            if (node.getQualifiedId().equals("/")) {
                return TreesTracker.getTracker().getProject(node.getTreeDB())
                        .getName();
            }
            return node.toString();
        }
        return WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider().getText(element);
    }

}
