/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.ui.dialogs;

import java.util.Collections;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

/**
 * Attribute edit dialog
 */
public class EditAttrDialog extends Dialog {
    ModifyListener updateDecorator = new ModifyListener() {
        @Override
        public void modifyText(ModifyEvent e) {
            String err = EditAttrDialog.this.getNameError(name.getText());
            if(getButton(Dialog.OK)!=null) {
                if (err != null) {
                    getButton(Dialog.OK).setEnabled(false);
                } else {
                    getButton(Dialog.OK).setEnabled(true);
                }
            }
            if (err != null) {
                attrNameDecorator.setDescriptionText(err);
                attrNameDecorator.show();
                return;
            }
            attrNameDecorator.hide();
        }
    };

    static String TITLE = "Attribute Editor";
    static String LBL_NAME = "Input a name of the attribute:";
    static String LBL_VALUE = "Input a value of the attribute:";

    static String MES_EXIST = "Attribute with same name already exists";
    static String MES_EMPTY = "Name of the attribute can not be empty";
    static String MES_WRONG_CHARS = "Spaces are not allowed";

    static int SINGLE_WIDTH = 267;
    ControlDecoration attrNameDecorator;
    private Text name = null;
    private Text value = null;

    private String[] initalVal = new String[2];
    private String[] curVal = new String[2];

    private Map<String, Object> attrs;
    private boolean createMode = true;

    private ControlDecoration createDecorator(Text text) {
        ControlDecoration controlDecoration = new ControlDecoration(text,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }

    private final Listener keyLink = new Listener() {
        public void handleEvent(Event event) {
            if ((event.type == SWT.KeyDown) && (event.keyCode == SWT.CR)) {
                if (name.isFocusControl() == true) {
                    value.setFocus();
                } else {
                    if (value.isFocusControl() == true)
                        okPressed();
                }
            }
        }
    };

    public EditAttrDialog(Shell shell, String name, String value,
            Map<String, Object> attrs) {
        this(shell, name, value, attrs, true);

    }
    public EditAttrDialog(Shell shell, String name, String value,
            Map<String, Object> attrs, boolean createMode) {
        super(shell);

        curVal[0] = name;
        curVal[1] = value;
        initalVal[0] = name;
        initalVal[1] = value;

        this.attrs = Collections.unmodifiableMap(attrs);
        this.createMode = createMode;
    }

    public String getKey() {
        return curVal[0];
    }

    public String getValue() {
        return curVal[1];
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = createClientContainer(parent, 1);
        Label nameLabel = new Label(container, SWT.NONE);
        nameLabel.setText(LBL_NAME);
        nameLabel.setLayoutData(new GridData(GridData.BEGINNING,
                GridData.CENTER, true, true));
        name = new Text(container, SWT.BORDER);
        name.setLayoutData(new GridData(SINGLE_WIDTH, -1));
        name.addListener(SWT.KeyDown, keyLink);
        attrNameDecorator = createDecorator(name);
        name.addModifyListener(updateDecorator);

        Label valueLabel = new Label(container, SWT.NONE);
        valueLabel.setText(LBL_VALUE);
        valueLabel.setLayoutData(new GridData(GridData.BEGINNING,
                GridData.CENTER, true, true));
        value = new Text(container, SWT.BORDER);
        value.setLayoutData(new GridData(SINGLE_WIDTH, -1));
        value.addListener(SWT.KeyDown, keyLink);
        value.addModifyListener(updateDecorator);

        name.setText(initalVal[0]);
        value.setText(initalVal[1]);

        this.getShell().setText(TITLE);
        this.getShell().setImage(
                PlatformUI.getWorkbench().getSharedImages()
                        .getImage(ISharedImages.IMG_OBJ_ELEMENT));
        return container;
    }

    private Composite createClientContainer(Composite parent, int span) {
        Composite container = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.marginWidth = layout.marginHeight = 10;
        layout.verticalSpacing = 2;
        layout.numColumns = span;
        container.setLayout(layout);
        return container;
    }

    @Override
    protected void okPressed() {
        String attrname = name.getText();
        curVal[0] = attrname;
        curVal[1] = value.getText();
        super.okPressed();
    }

    protected String getNameError(String attrname) {
        if ((initalVal[0].equals(attrname) == false)) {
            for (String a : attrs.keySet()) {
                if (a.equals(attrname) == true && createMode) {
                    return MES_EXIST;
                }
            }
        }

        if (attrname.trim().length() != attrname.length()) {
            return MES_WRONG_CHARS;
        }
        if (attrname.length() == 0) {
            return MES_EMPTY;
        }

        return null;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
                false).setEnabled(false);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }
}