package com.unitesk.requality.eclipse.ui.dialogs;

import java.util.ArrayList;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.eclipse.editors.browser.ReqDocBrowser;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;

/**
 * Find dialog
 */
public class FindDialog extends Dialog {
    private static String TITLE = "Find";
    private static String LBL_FIND = "Find:";
    private ReqDocBrowser reqDocBrowser;
    private Combo combo;
    public static ArrayList<String> comboStrings = new ArrayList<String>();
    
    private Button caseSensitive;
    private static boolean selectCaseSensitive = false;
    private SelectionListener caseSensitiveListener = new SelectionListener() {
        
        @Override
        public void widgetSelected(SelectionEvent e) {
            selectCaseSensitive = caseSensitive.getSelection();
        }
        
        @Override
        public void widgetDefaultSelected(SelectionEvent e) {}
    };
    
//    private Button wrapAround;
//    private static boolean selectWrapAround = false;
//    private SelectionListener wrapAroundListener = new SelectionListener() {
//        
//        @Override
//        public void widgetSelected(SelectionEvent e) {
//            selectWrapAround = wrapAround.getSelection();
//        }
//        
//        @Override
//        public void widgetDefaultSelected(SelectionEvent e) {}
//    };
    
    private ModifyListener comboListener = new ModifyListener() {

        @Override
        public void modifyText(ModifyEvent e) {
            if (combo.isDisposed())
                return;
            if (combo.getText().isEmpty()) {
                getButton(IDialogConstants.CLIENT_ID + 1).setEnabled(false);
                getButton(IDialogConstants.CLIENT_ID + 2).setEnabled(false);
            } else {
                getButton(IDialogConstants.CLIENT_ID + 1).setEnabled(true);
                getButton(IDialogConstants.CLIENT_ID + 2).setEnabled(true);
            }

        }
    };

    public FindDialog(Shell shell, ReqDocBrowser rdBrowser) {
        super(shell);
        setShellStyle(SWT.DIALOG_TRIM | SWT.CLOSE | SWT.MODELESS | SWT.BORDER
                | SWT.TITLE);
        reqDocBrowser = rdBrowser;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        getShell().setText(TITLE);
        Composite comp1 = new Composite(parent, SWT.NONE);
        comp1.setLayout(new GridLayout(2, false));
        Composite comp3 = new Composite(comp1, SWT.NONE);
        comp3.setLayout(new GridLayout(1, false));
        GridData gd = new GridData();
        gd.verticalAlignment = GridData.BEGINNING;
        Label label = new Label(comp3, SWT.TOP);
        label.setText(LBL_FIND);
        comp3.setLayoutData(gd);
        Composite comp4 = new Composite(comp1, SWT.NONE);
        comp4.setLayout(new GridLayout());
        combo = new Combo(comp4, SWT.NONE);
        GridData gdCombo = new GridData();
        gdCombo.widthHint = 400;
        combo.setLayoutData(gdCombo);
        if (!comboStrings.isEmpty()) {
            combo.setItems(comboStrings.toArray(new String[0]));
            combo.setText(comboStrings.get(0));
        }
        combo.setText(reqDocBrowser.getSelectionText());
        combo.addModifyListener(comboListener);
        caseSensitive = new Button(comp4, SWT.CHECK);
        caseSensitive.setText("Case sensitive");
        caseSensitive.addSelectionListener(caseSensitiveListener);
        caseSensitive.setSelection(selectCaseSensitive);
//        wrapAround = new Button(comp4, SWT.CHECK);
//        wrapAround.setText("Wrap search");
//        wrapAround.addSelectionListener(wrapAroundListener);
//        wrapAround.setSelection(selectWrapAround);
        return parent;
    }

    protected void createButtonsForButtonBar(Composite parent) {
        createButton(parent, IDialogConstants.CANCEL_ID, "Close", false);
        Button spaceButton = createButton(parent, 3, "", false);
        spaceButton.setVisible(false);
        createButton(parent, IDialogConstants.CLIENT_ID + 1, "<<", false);
        createButton(parent, IDialogConstants.CLIENT_ID + 2, ">>", true);
//        if (comboStrings.isEmpty()) {
        if (combo.getText().isEmpty()) {
            getButton(IDialogConstants.CLIENT_ID + 1).setEnabled(false);
            getButton(IDialogConstants.CLIENT_ID + 2).setEnabled(false);
        }
    }

    @Override
    protected void buttonPressed(int buttonId) {
        super.buttonPressed(buttonId);
        if (combo.isDisposed())
            return;
        if (combo.getText().isEmpty())
            return;
        saveTextToCombo();
        IEditorPart focusedEditor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
        if(focusedEditor != null && focusedEditor instanceof ReqMarker) {
            reqDocBrowser = ((ReqMarker) focusedEditor).getBrowser();
        }
        
        if (IDialogConstants.CLIENT_ID + 1 == buttonId) {
            reqDocBrowser.findText(combo.getText(),
                    selectCaseSensitive, true,
                    false);
//                    selectWrapAround);
        }
        if (IDialogConstants.CLIENT_ID + 2 == buttonId) {
            reqDocBrowser.findText(combo.getText(),
                    selectCaseSensitive, false,
                    false);
//                    selectWrapAround);
        }
    }

    private void saveTextToCombo() {
        String text = combo.getText();
        if (comboStrings.contains(text)) {
            comboStrings.remove(text);
        }
        comboStrings.add(0, text);
        combo.setItems(comboStrings.toArray(new String[0]));
        combo.setText(text);
    }

}
