/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.ui.dialogs;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.panels.AttributesPanel;
import com.unitesk.requality.eclipse.editors.panels.SelectNodePanel;
import com.unitesk.requality.eclipse.editors.panels.SelectRequirementPanel;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.report.Template;

public class ReportGenDialog extends Dialog {
    private static final int SAVE_SETTINGS_ID = IDialogConstants.CLIENT_ID + 1;
    private static Image ICON;
    private static String TITLE = "Report Generator";

    private static String LBL_ROOT = "Root requirement:";
    private static String LBL_TEMPLATE = "Template:";
    private static String LBL_SAVE = "Save to:";

    // Widgets
    private SelectRequirementPanel rootReqPanel;
    private AttributesPanel attrPanel;
    private SelectNodePanel outNodePanel;
    private Combo templates;

    private Requirement req = null;
    private String savePath = "";
    private LinkedHashMap<String, Template> tpls = new LinkedHashMap<String, Template>();
    private TreeDB db = null;

    private Listener dataChangeListener = new Listener() {
        @Override
        public void handleEvent(Event event) {
            updateUI();
        }
    };
    private String selectedTemplateId;
    private TreeNode initalAttrsNode;

    public ReportGenDialog(Shell shell, TreeDB db) {
        super(shell);
        this.db = db;
        if (db.getNode("/Reports") == null) {
            db.getRootNode().createChild("Reports", "ReportFolder");
        }
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        ICON = PlatformUI.getWorkbench().getSharedImages()
                .getImage(ISharedImages.IMG_OBJ_ELEMENT);
        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(2, false));
        GridData layData = new GridData(GridData.FILL_BOTH);
        layData.widthHint = 800;
        layData.heightHint = 400;
        container.setLayoutData(layData);

        new Label(container, SWT.NONE).setText(LBL_ROOT);
        rootReqPanel = new SelectRequirementPanel(container, SWT.NONE, db);
        rootReqPanel.setCanSelectRoot(true);
        if (req == null) {
            req = (Requirement) db.getNode(Requirement.getTypeRootQId());
        }
        rootReqPanel.setRequirement(req);

        new Label(container, SWT.NONE).setText(LBL_TEMPLATE);
        createSelectTemplatePanel(container);

        attrPanel = new AttributesPanel(container, SWT.NONE);
        GridData data = new GridData(GridData.FILL_BOTH);
        data.horizontalSpan = 2;
        attrPanel.setLayoutData(data);

        IFolder repFolder = TreesTracker.getTracker().getProject(db)
                .getFolder("reports");
        if (!repFolder.exists()) {
            try {
                repFolder.create(true, true, null);
            } catch (CoreException e) {
                e.printStackTrace();
            }
        }

        new Label(container, SWT.NONE).setText(LBL_SAVE);
        savePath = repFolder.getLocation().toOSString();
        Composite savePanel = new Composite(container, SWT.NONE);
        savePanel.setLayout(new GridLayout(2, true));
        savePanel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        SelectReportDataDialog rddialog = new SelectReportDataDialog(
                getParentShell(), db);
        outNodePanel = new SelectNodePanel(savePanel, SWT.NONE, db, rddialog);

        if (initalAttrsNode != null) {
            attrPanel.setAttributes(initalAttrsNode);
            outNodePanel.setNode(initalAttrsNode);
        }

        rootReqPanel.addListener(SWT.Modify, dataChangeListener);
        templates.addListener(SWT.Modify, dataChangeListener);

        outNodePanel.addListener(SWT.Modify, dataChangeListener);
        this.getShell().setText(TITLE);
        this.getShell().setImage(ICON);
        return container;
    }

    private Composite createSelectTemplatePanel(Composite root) {
        Composite panel = new Composite(root, SWT.NONE);
        panel.setLayout(new GridLayout());
        templates = new Combo(panel, SWT.NONE | SWT.READ_ONLY);
        tpls = new LinkedHashMap<String, Template>(Activator.getDefault()
                .getReportTemplates());
        if (tpls.size() == 0) {
            return panel;
        }
        if (selectedTemplateId == null) {
            selectedTemplateId = tpls.keySet().iterator().next();
        }

        for (Template tpl : tpls.values()) {
            templates.add(tpl.getName());
        }
        Iterator<String> it = tpls.keySet().iterator();
        String curId = "";
        int i = 0;
        for (; i < tpls.size() && !selectedTemplateId.equals(curId); i++) {
            curId = it.next();
        }
        templates.select(i - 1);
        return panel;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
//        createButton(parent, SAVE_SETTINGS_ID, "Save settings", false);
        super.createButtonsForButtonBar(parent);
        updateUI();
    }

    private void updateUI() {
        Button okButton = this.getButton(IDialogConstants.OK_ID);
        okButton.setEnabled(isDataCorrect());

        Iterator<String> it = tpls.keySet().iterator();
        for (int i = 0; i < templates.getSelectionIndex() + 1; i++) {
            selectedTemplateId = it.next();
        }
    }

    private boolean isDataCorrect() {
        if (rootReqPanel.getSelectedRequirement() == null)
            return false;
        if (tpls.size() == 0)
            return false;
        if (!isReportDataNodeExist()) {
            return false;
        }
        return true;
    }

    private boolean isReportDataNodeExist() {
        if (outNodePanel.getSelectedNode() == null)
            return false;
        return outNodePanel.getSelectedNode().getType().equals("ReportData");
    }

    public Map<String, Object> getData() {
        return attrPanel.getAttributes();
    }

    public Template getTemplate() {
        return tpls.get(selectedTemplateId);
    }

    public String getSaveDir() {
        return savePath;
    }

    public void setStartNode(Requirement req) {
        this.req = req;
    }

    @Override
    protected void buttonPressed(int buttonId) {
        if (buttonId == OK) {
            TreeNode node = outNodePanel.getSelectedNode();
            savePath = db.getStorage().getNodeResourcePath(node.getUUId());
            saveReportSettiong(node);
        }
        super.buttonPressed(buttonId);
        if (buttonId == SAVE_SETTINGS_ID) {
            InputDialog dialog = new InputDialog(getParentShell(),
                    "Input Node Name", "Input new report settrings node id",
                    req.getId() + "-" + getTemplate().getName(), null);
            if (dialog.open() == Dialog.OK) {
                TreeNode node = db.getNode("/Reports").createChild(dialog.getValue(),
                        "ReportData");
                saveReportSettiong(node);
                outNodePanel.setNode(node);
                updateUI();
            }
        }
    }

    private void saveReportSettiong(TreeNode node) {
        Map<String, Object> data = getData();
        for (String key : data.keySet()) {
            node.putAttribute(new Attribute(AttributeType.STRING, key, data
                    .get(key)));
        }
        node.putAttribute(new Attribute(AttributeType.STRING,
                "_root_requiremnt_qid", req.getQualifiedId()));
        node.putAttribute(new Attribute(AttributeType.STRING, "_template_id",
                getTemplate().getId()));
        node.saveAttributes();
    }

    public void setTemplateId(String templateId) {
        this.selectedTemplateId = templateId;
    }

    public void setInitalData(TreeNode attrs) {
        initalAttrsNode = attrs;
    }
}
