/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.editors;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.UUID;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeMapper;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.AddSelectionToRequirement;
import com.unitesk.requality.nodetypes.Requirement;

public class UniEditor extends EditorPart {

    public static final String ID = "com.unitesk.requality.editors.UniEditor";
    private Browser browser;
    private TreeNode node;
    private TreeDB db;

    public UniEditor() {
    }

    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    @Override
    public void doSaveAs() {
    }

    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        setSite(site);
        setInput(input);
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    protected String node2jsonString(TreeNode n) throws JSONException {
        String result;
        JSONObject json = NodeMapper.toJSON(n);
        json.put("id", n.getId());
        JSONArray jsonChildren = new JSONArray();
        TreeNode[] children = n.getSortedChildren();
        for (int i = 0; i < children.length; i++) {
            jsonChildren.put(children[i].getUUId().toString());
        }
        json.put("children", jsonChildren);

        if (n.getQualifiedId().equals(Requirement.getTypeRootQId())) {
            JSONObject attr = json.getJSONObject("attributes");
            @SuppressWarnings("rawtypes")
            Iterator keys = attr.keys();
            while(keys.hasNext()) {
                String key = (String) keys.next();
                JSONObject a = attr.getJSONObject(key);
                if (a.getString("key").equals("_type")) {
                    // TODO: check this
                    a.putOpt("value", null);
                }
            }
        }
        result = json.toString(4);
        return result;
    }

    @Override
    public void createPartControl(Composite parent) {
        browser = new Browser(parent, SWT.MOZILLA);
        try {
            node = ((NodeEditorInput) getEditorInput()).getStorage().getNode();
            db = node.getTreeDB();
            File html = new File(Activator.getPath("jstools/unieditor.html"));
            browser.setUrl(html.toURI().toURL().toString());

            new BrowserFunction(browser, "getNodeByUUID") {
                @Override
                public Object function(Object[] arguments) {
                    UUID nodeUUID = UUID.fromString((String) arguments[0]);
                    TreeNode n = db.getNode(nodeUUID);
                    String result;
                    try {
                        result = node2jsonString(n);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                    return result;
                }
            };

            new BrowserFunction(browser, "getNodeByQID") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode((String) arguments[0]);
                    String result;
                    try {
                        result = node2jsonString(n);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                    return result;
                }
            };
            /*
             * Request for the root element of the selected subtree Input
             * parameters: none Return value: String value containing uuid of
             * the subtree root node
             */
            new BrowserFunction(browser, "getStartNode") {
                @Override
                public Object function(Object[] arguments) {
                    return node.getUUId().toString();
                }
            };

            /*
             * Request to change the node ID Input parameters: uuid - (String)
             * uuid of the updated node new_id - (String) new ID to be set
             * Return value: Boolean value: true if operation was successful,
             * false otherwise
             */
            new BrowserFunction(browser, "changeNodeId") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String newId = (String) arguments[1];
                    n.setId(newId);
                    return true;
                }
            };

            new BrowserFunction(browser, "changeNodeName") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    if (!n.getType().equals("Requirement")) {
                        return false;
                    }
                    String newName = (String) arguments[1];
                    ((Requirement) n).setName(newName);
                    n.saveAttributes();
                    return true;
                }
            };

            /*
             * Request to change the value of one of the node attributes Input
             * parameters: uuid - (String) uuid of the updated node attr_name -
             * (String) name of the changed attribute new_val - (String) new
             * attribute value to be set Return value: Boolean value: true if
             * operation was successful, false otherwise
             */
            new BrowserFunction(browser, "changeNodeAttr") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String key = (String) arguments[1];
                    String value = (String) arguments[2];
                    Attribute attr = new Attribute(AttributeType.STRING, key,
                            value);
                    n.putAttribute(attr);
                    n.saveAttributes();
                    return true;
                }
            };

            /*
             * Request to change the move a node into different subtree Input
             * parameters: uuid - (String) uuid of the moved node
             * new_parent_uuid - (String) uuid of the new parent node Return
             * value: Boolean value: true if operation was successful, false
             * otherwise
             */
            new BrowserFunction(browser, "changeNodeParent") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    TreeNode parent = db.getNode(UUID
                            .fromString((String) arguments[1]));
                    if (!n.getParent().equals(parent)) {
                        n.setParent(parent);
                        return true;
                    }
                    return false;
                }
            };

            /*
             * Request to insert a new node Input parameters: uuid - (String)
             * uuid of the new node parent_uuid - (String) uuid of the parent
             * node Return value: Boolean value: true if operation was
             * successful, false otherwise
             */
            new BrowserFunction(browser, "addNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode parent = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String id = (String) arguments[1];
                    if(id.length()==0) {
                        id = AddSelectionToRequirement.getNextId(parent);
                    }
                    String type = (String) arguments[2];
                    if (parent.findChild(id) == null) {
                        TreeNode newNode = parent.createChild(id, type);
                        return newNode.getUUId().toString();
                    }
                    return null;
                }
            };

            /*
             * Request to remove a node with all its decendants Input
             * parameters: uuid - (String) uuid of the removed node Return
             * value: Boolean value: true if operation was successful, false
             * otherwise
             */
            new BrowserFunction(browser, "removeNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    n.delete();
                    return true;
                }
            };

        } catch (CoreException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        db.addListener(treeChangeListener);
    }

    @Override
    public void setFocus() {
    }

    public void dispose() {
        db.removeListener(treeChangeListener);
        super.dispose();
    };

    ITreeChangeListener treeChangeListener = new ITreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
        }

        @Override
        public void deleted(TreeNode node) {
            refresh(node.getParent());
        }

        @Override
        public void changed(TreeNode node) {
            refresh(node);
        }

        @Override
        public void added(TreeNode node) {
            refresh(node.getParent());
        }
    };

    protected void refresh(final TreeNode node) {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if(browser!=null && !browser.isDisposed()) {
                    browser.execute("onChangeNode('" + node.getUUId() + "');");
                }
            }
        });

    }
}
