/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.tools.TreesTracker;

public class TreesContentProvider implements ITreeContentProvider {
    private String rootQId = "/";
    private String[] supportedTypes;

    public TreesContentProvider() {
    }

    public void setSupportedTypes(String[] supportedTypes) {
        this.supportedTypes = supportedTypes;
    }

    public boolean isTypeSupported(String type) {
        if(supportedTypes==null) {
            return true;
        }
        for(String t: supportedTypes) {
            if(type.equals(t)) {
                return true;
            }
        }
        return false;
    }

    public TreesContentProvider(String rootQId) {
        this.rootQId = rootQId;
    }


    public String getRootQId() {
        return rootQId;
    }

    @Override
    public Object[] getElements(Object inputElement) {
        if (inputElement instanceof TreeDB) {
            TreeDB db = (TreeDB) inputElement;
            TreeNode node = db.getNode(getRootQId());
            if (node != null) {
                return new Object[] { node };
            }
            return new Object[] {};
        }

        if (inputElement instanceof IWorkspaceRoot) {
            try {
                TreeDB[] trees = TreesTracker.getTracker().getTrees();
                List<TreeNode> result = new ArrayList<TreeNode>();
                for (TreeDB db : trees) {
                    TreeNode root = db.getNode(getRootQId());
                    if (root != null) {
                        result.add(root);
                    }
                }
                return result.toArray();
            } catch (Exception e) {
                e.printStackTrace();
                return new Object[] {};
            }
        }

        throw new IllegalArgumentException();
    }

    @Override
    public Object[] getChildren(Object parentElement) {
        TreeNode parent = null;
        if (parentElement instanceof TreeDB) {
            TreeDB treedb = (TreeDB) parentElement;
            parent = (TreeNode) treedb.getNode(getRootQId());
        }
        if (parentElement instanceof TreeNode) {
            parent = (TreeNode) parentElement;
        }

        ArrayList<Object> result = new ArrayList<Object>();
        if (parent != null) {
            for (int i = 0; i < parent.sizeChildren(); i++) {
                if(isTypeSupported(parent.getChild(i).getType())) {
                    result.add(parent.getChild(i));
                }
            }
        }
        return result.toArray();
    }

    @Override
    public Object getParent(Object element) {
        if (element instanceof TreeNode) {
            TreeNode node = (TreeNode) element;
            return node.getParent();
        }
        return null;
    }

    @Override
    public boolean hasChildren(Object element) {
        if (element instanceof TreeDB) {
            TreeDB db = (TreeDB) element;
            TreeNode root = (TreeNode) db.getNode(getRootQId());
            if ((root != null) && (root.sizeChildren() > 0))
                return true;
        }

        if (element instanceof TreeNode) {
            TreeNode node = (TreeNode) element;
            if ((node != null) && (node.sizeChildren() > 0)) {
                return getChildren(element).length>0;
            }
        }
        return false;
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
    }

    @Override
    public void dispose() {
    }
}
