/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.documents;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.documents.IDocumentProcessor;
import com.unitesk.requality.nodetypes.Document;

public abstract class DocumentProcessor implements IDocumentProcessor {

    private String encoding = "UTF-8";

    @Override
    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    @Override
    public void process(String docQId, TreeNode rootNode) throws IOException {
        try {
            TreeDB db = rootNode.getTreeDB();
            Document doc = (Document) db.getNode(docQId);
            String docPath = doc.getHTMLPath();
            IFile docResNode = ResourcesPlugin.getWorkspace().getRoot()
                    .getFileForLocation(new Path(docPath));
            
            Reader reader = new InputStreamReader(docResNode.getContents(), encoding);
            
            ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
            Writer writer = new OutputStreamWriter(byteOut, encoding);

            process(docQId, reader, writer, rootNode);
            writer.close();
            reader.close();

            docResNode.setContents(new ByteArrayInputStream(byteOut.toByteArray()), false, true, null);
        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    protected abstract void process(String docId, Reader reader, Writer writer,
            TreeNode rootNode) throws IOException;

    @Override
    public boolean configure(Document doc) {
        return true;
    }
}
