/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.documents;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.UUID;

import javax.xml.stream.XMLEventFactory;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.events.XMLEvent;

import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class DummyDocumentProcessor extends DocumentProcessor {
    private static int limit = 100;

    public static void setLimit(int lim) {
	limit = lim;
    }

    @Override
    protected void process(String docId, Reader in, Writer out,
	    TreeNode rootNode) throws IOException {
	InputDialog lim = new InputDialog(PlatformUI.getWorkbench()
		.getActiveWorkbenchWindow().getShell(), "Location limit",
		"Set limit:", null, null);
	lim.open();
	setLimit(new Integer(lim.getValue()));
	try {
	    XMLEventReader reader = XMLInputFactory.newInstance()
		    .createXMLEventReader(in);
	    XMLEventWriter writer = XMLOutputFactory.newInstance()
		    .createXMLEventWriter(out);
	    XMLEventFactory eventFactory = XMLEventFactory.newInstance();

	    int chIndex = 0;
	    int count = 0; // TODO: remove

	    Requirement req = null;
	    Location location = null;
	    String locationId = null;

	    String name = null;

	    while (reader.hasNext()) {
		XMLEvent event = (XMLEvent) reader.next();
		if (count > limit)
		    writer.add(event);
		else if (event.isCharacters()) {
		    String data = event.asCharacters().getData();

		    Requirement parent = (Requirement) rootNode;
		    req = (Requirement) parent.createChild(UUID.randomUUID()
			    .toString());
		    Document doc = (Document) req.getTreeDB().getNode(docId);
		    location = (Location) doc.createChild(UUID.randomUUID()
			    .toString(), "Location");
		    locationId = location.getId().toString();
		    req.addLocation(location.getQualifiedId());
		    req.setName(UUID.randomUUID().toString());
		    req.saveAttributes();

		    count++;

		    if (chIndex == 0) {
			if (data.trim().length() > 0)
			    chIndex++;
		    } else {
			chIndex++;
		    }
		    if (chIndex > 0) {
			name += data;

			writer.add(eventFactory.createStartElement("", "",
				"span"));
			writer.add(eventFactory.createAttribute("class",
				"requality_text id_" + locationId));

			if (chIndex == 1) {
			    writer.add(eventFactory.createStartElement("", "",
				    "a"));
			    writer.add(eventFactory.createAttribute("name",
				    locationId));
			    writer.add(eventFactory.createAttribute("id", "id_"
				    + locationId));
			    writer.add(eventFactory.createAttribute("class",
				    "requality_id"));
			    writer.add(eventFactory.createEndElement("", "",
				    "a"));
			}

			writer.add(event);

			writer.add(eventFactory
				.createEndElement("", "", "span"));
		    } else {
			writer.add(event);
		    }
		    chIndex++;
		} else {
		    writer.add(event);
		}
	    }
	} catch (Throwable e) {
	    throw new IOException(e);
	}
    }
}
