/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.tools.dom;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;

import com.unitesk.requality.tools.RequalityCLI;

public class XHTMLResourceCollector {
    private final IXHTMLNode root;
    private final String src;
    private final String dst;
    private List<String> procesedResoureces = new ArrayList<String>();

    public XHTMLResourceCollector(IXHTMLNode root, String srcHtmlFolder,
            String dst) {
        this.root = root;
        this.src = srcHtmlFolder;
        this.dst = dst;
    }

    public void process() {
        walk(root);
    }

    private void walk(IXHTMLNode node) {
        proccess(node);
        for (IXHTMLNode child : node.getChildren()) {
            walk(child);
        }
    }

    private void proccess(IXHTMLNode node) {
        try {
            if (node.getName().toLowerCase().equals("img")) {
                if (node.getAttributeValue("src") != null
                        && !procesedResoureces.contains(node
                                .getAttributeValue("src"))) {
                    procesedResoureces.add(node.getAttributeValue("src"));
                    try {
                        String newPath = copyResource(
                                node.getAttributeValue("src"), "imgs");
                        node.setAttribute("src", newPath);
                    } catch (Exception e) {
                        System.err.println(e.getMessage());
                    }
                }
            } else if (node.getName().toLowerCase().equals("object")) {
                if (node.getAttributeValue("data") != null
                        && !procesedResoureces.contains(node
                                .getAttributeValue("data"))) {
                    procesedResoureces.add(node.getAttributeValue("data"));
                    try {
                        String newPath = copyResource(
                                node.getAttributeValue("data"), "imgs");
                        node.setAttribute("data", newPath);
                    } catch (Exception e) {
                        System.err.println(e.getMessage());
                    }
                }
            }
            else if (node.getName().toLowerCase().equals("link")) {
                if ("stylesheet".equals(node.getAttributeValue("rel"))
                        && node.getAttributeValue("href") != null
                        && !procesedResoureces.contains(node
                                .getAttributeValue("href"))) {
                    procesedResoureces.add(node.getAttributeValue("href"));
                    String newPath;
                    newPath = copyResource(node.getAttributeValue("href"),
                            "css");
                    node.setAttribute("href", newPath);
                }
            } else if (node.getName().toLowerCase().equals("script")) {
                if (node.getAttributeValue("src") != null
                        && !procesedResoureces.contains(node
                                .getAttributeValue("src"))) {
                    procesedResoureces.add(node.getAttributeValue("src"));
                    String newPath = copyResource(
                            node.getAttributeValue("src"), "js");
                    node.setAttribute("src", newPath);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String copyResource(String encodedDocImgSrc, String subfolder)
            throws Exception {
        String docImgSrc = java.net.URLDecoder
                .decode(encodedDocImgSrc, "utf-8");
        String imgNewPath = dst + File.separator + subfolder + File.separator;
        String dstFile = getDstFileName(imgNewPath,
                new File(docImgSrc).getName());
        new File(imgNewPath).mkdirs();

        if (docImgSrc.startsWith("http://") || docImgSrc.startsWith("https://")) {
            URL srcUrl = new URL(docImgSrc);
            InputStream in = srcUrl.openConnection().getInputStream();
            FileOutputStream out = new FileOutputStream(URLDecoder.decode(
                    dstFile, "utf-8"));
            int inp;
            while ((inp = in.read()) != -1) {
                out.write(inp);
            }
        } else if(docImgSrc.startsWith("file:/")) {
            RequalityCLI.copyfile(new File(new URI(encodedDocImgSrc)).getAbsolutePath(), dstFile);
        }
        else {
            String sourcePath = src + File.separator + docImgSrc;
            RequalityCLI.copyfile(sourcePath, dstFile);
        }
        return subfolder + "/" + new File(dstFile).getName();
    }

    private String getDstFileName(String imgNewPath, String name) {
        if (!new File(imgNewPath + name).exists()) {
            return imgNewPath + name;
        }
        do {
            name = "_" + name;
        } while (new File(name).exists());
        return imgNewPath + name;
    }
}
